@@ -4,15 +4,19 @@ package RDBAL::Config;
 $cache_directory = '/usr/local/schema_cache';
 
 # Preferred order of middle layer drivers
-@search_order = ('ApacheSybaseDBlib',
-		 'SybaseDBlib',
-		 'ODBC',
+@search_order = (
 		 'dbi:Sybase',
 		 'dbi:Oracle',
-		 'dbi:mSQL'
+		 'Pg',
+		 'dbi:mSQL',
+		 'ODBC',
+		 'ApacheSybaseDBlib',
+		 'SybaseDBlib'
 		 );
 # Perl module for each middle layer driver
-%middle_module = ('ApacheSybaseDBlib' => 'Apache::Sybase::DBlib',
+%middle_module = (
+		  'Pg' => 'Pg',
+		  'ApacheSybaseDBlib' => 'Apache::Sybase::DBlib',
 		  'SybaseDBlib' => 'Sybase::DBlib',
 		  'ODBC' => 'Win32::ODBC',
 		  'dbi:' => 'DBI'
